#ifndef MPEGPLAYER_H
#define MPEGPLAYER_H

#include <DirectWindow.h>
#include <Locker.h>
#include <libmpeg3.h>
#include "_MediaBar_.h"

class AudioPlayer {
public:
	AudioPlayer(mpeg3_t *file, int track, VideoWindow* owner);
	
	virtual ~AudioPlayer();

public:
	status_t InitCheck(void);

	bigtime_t CurrentTime() const;
	
	bigtime_t PerformanceTime() const;
	
	bigtime_t Latency() const;

	media_raw_audio_format Format() const;
	
	float Volume() const;
	
	void SetVolume(float volume);
	
	bool Repeat() const;
	
	void SetRepeat(bool repeat);
	
	void SetTrack(int track);
	
	status_t Start();
	
	void Stop();
	
	bool HasData() const;

public:
	long CountFrames() const;
	
	long CurrentFrame() const;
	
	bigtime_t Duration() const;
	
	bigtime_t CurrentTrackTime() const;
	
	void SeekToFrame(long frame);
	
	void SeekToTime(bigtime_t time);
	
	void ReadFrames(void *outFrames, int frameCount);
	
private:
	static void PlayBuffer(void *cookie, void *buffer, size_t size,
						   const media_raw_audio_format & format);
	
private:
	VideoWindow* fOwner;
	BSoundPlayer *fPlayer;
	mpeg3_t	*fFile;
	int fTrack;
	bool fRepeat;
	status_t fInitCheck;
};

struct blit_rect {
	int src_left, src_top;
	int src_width, src_height;
	int dst_left, dst_top;
	int dst_width, dst_height;
};

class VideoPlayer {
public:
	VideoPlayer(mpeg3_t *file, int track, VideoWindow* owner);
	
	virtual ~VideoPlayer();

public:
	bigtime_t CurrentTime();

	bigtime_t PerformanceTime();
	
	bigtime_t Latency();

	media_raw_video_format Format();
	
	int Width();
	
	int Height();

	status_t Start();
	
	void Stop();

	bool Repeat() const;
	
	void SetRepeat(bool repeat);
	
	void SetDrawBuffer(void (*drawBuffer)(void *), void *cookie);
	
	void SetAudioPlayer(AudioPlayer*);
public:
	long CountFrames();
	
	long CurrentFrame();
	
	bigtime_t Duration();
	
	void SeekToFrame(long frame);
	
	void SeekToTime(bigtime_t time);

	void ReadFrame(unsigned char **outFrame, color_space space,
				   const blit_rect *rects, int count);

	void DropFrames(int frames);
	
private:
	void Run();

	static int32 DrawBuffer(void *cookie);

private:
	VideoWindow* fOwner;
	BLocker fLocker;
	mpeg3_t	*fFile;
	int fTrack;
	bool fRepeat;
	volatile bigtime_t fCurrentTime;
	bigtime_t fLatency;
	thread_id fThread;
	volatile bool fConnected;
	volatile void (*fDrawBuffer)(void *);
	void *fCookie;
	AudioPlayer *fAudioPlayer;
	long frame;
	bigtime_t startTime;
};


class VideoWindow : public BDirectWindow {
public:
	VideoWindow(BRect frame, const char *title, window_type type, uint32 flags);
	
	virtual ~VideoWindow();

	virtual void MessageReceived(BMessage *message);
		
	virtual void DirectConnected(direct_buffer_info *info);

	void OpenFile(entry_ref *ref);

	void CloseFile();

	virtual void	MouseDown(BPoint where);

	void Start();
	
	void Stop();

	void SetScale(float scale);
	
	void SetRatio(float ratio);

	bool Repeat();
	
	void SetRepeat(bool repeat);
	
	bool Full();
	
	void SetFull(bool full);
	
	bool IsPlaying() const;
	
	status_t Play();

	status_t StopPlaying();
	
	status_t Control(media_action action);
	
	bigtime_t CurrentTime();
	
	void SeekToTime(bigtime_t time);
	
	void SetAudioTrack(int track);
	
	void Draw();

	void CreateBuffer(direct_buffer_info *info, int width, int height, float menuBarHeight);
	
	void DeleteBuffer();
	
private:
	static void DrawBuffer(void *cookie);

private:
	friend class _MediaSlider_;
	
	BLocker fLocker;
	float fMenuBarHeight;

	BMenuBar *fMenuBar;
	BView *fView;
	mpeg3_t *fFile;
	mpeg3_t *fFileAudio;
	int fLastTrack;
public:
	VideoPlayer *fVideoPlayer;
	AudioPlayer *fAudioPlayer;
	_MediaBar_*		fMediaBar;
	
private:	bigtime_t		fScrubTime;
	sem_id			fScrubSem;
public: sem_id			fPlaySem;
	
private:	bool fDirty;
	bool fConnected;
	bool fPlaying;
	direct_buffer_info *fInfo;
	
	color_space fBufferPixelFormat;
	int fBufferNumClips;
	unsigned char **fBufferRows;
	blit_rect *fBufferClips;
};

#endif
